@@ -250,6 +250,9 @@ class FeedManager(gobject.GObject):
             if updated_feed_date > feed_date:
                 # Merge feed data
                 old_feed_entries = feed_data.entries[:]
+                for entry in updated_feed_data.entries:
+                    if entry in old_feed_entries:
+                        updated_feed_data.remove(entry)
                 feed_data.update(updated_feed_data)
                 feed_data.entries += old_feed_entries
 
