@@ -29,7 +29,7 @@ import hildon
 import pygtk
 pygtk.require('2.0')
 import gtk
-import gtkhtml2
+import webkit
 import os
 import urllib2
 from portrait import FremantleRotation
@@ -39,10 +39,19 @@ from feedmanager import FeedManager
 import re, htmlentitydefs
 from threading import Thread
 import time
+from settings import Settings
 
 _ = gettext.gettext
 
+# Pre-cache colors
+color_style = gtk.rc_get_style_by_paths(gtk.settings_get_default() , 'GtkButton', 'osso-logical-colors', gtk.Button)
+active_color = color_style.lookup_color(constants.ACTIVE_TEXT_COLOR)
+default_color = color_style.lookup_color(constants.DEFAULT_TEXT_COLOR)
+secondary_color = color_style.lookup_color(constants.SECONDARY_TEXT_COLOR)
+del color_style
+
 manager = FeedManager()
+settings = Settings()
 
 # From http://effbot.org/zone/re-sub.htm#unescape-html
 def unescape(text):
@@ -146,34 +155,65 @@ class FeedsView(hildon.GtkTreeView):
         else:
             url = feed_data.href
 
-        color = self.style.lookup_color(constants.SECONDARY_TEXT_COLOR).to_string()
         return '%s\n<span face="monospace" foreground="%s"' \
             'size="x-small">%s</span>' % \
-            (feed_data.feed.title, color, url)
+            (feed_data.feed.title, secondary_color.to_string(), url)
 
+    def sort(self, order):
+        if (order == constants.ASCENDING_ORDER):
+            self.get_model().set_sort_column_id(self.FEEDS_NAME_COLUMN,
+                                                gtk.SORT_ASCENDING)
+        else:
+            self.get_model().set_sort_column_id(self.FEEDS_NAME_COLUMN,
+                                                gtk.SORT_DESCENDING)
 
-class PreferencesDialog(gtk.Dialog):
+class SettingsDialog(gtk.Dialog):
 
     def __init__(self, parent):
-        super(PreferencesDialog, self).__init__(_('Settings'), parent,
+        super(SettingsDialog, self).__init__(_('Settings'), parent,
                                                 gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                                 (gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
 
-
+        # Orientation picker
         self.selector_orientation = hildon.TouchSelector(text=True)
         for caption in FremantleRotation.MODE_CAPTIONS:
             self.selector_orientation.append_text(caption)
-        self.selector_orientation.set_active(0, FremantleRotation.AUTOMATIC)
-        self.picker_orientation = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
+        self.picker_orientation = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
         self.picker_orientation.set_selector(self.selector_orientation)
         self.picker_orientation.set_alignment(0, 0.5, 0, 0)
         self.picker_orientation.set_title(_("Screen orientation"))
-#        self.picker_orientation.connect("value-changed", on_picker_value_changed)
-
         self.vbox.add(self.picker_orientation)
         self.picker_orientation.show()
 
+        # Font size picker
+        self.selector_font_size = hildon.TouchSelector(text=True)
+        for i in constants.font_size_range:
+            self.selector_font_size.append_text(str(i))
+        self.picker_font_size = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        self.picker_font_size.set_selector(self.selector_font_size)
+        self.picker_font_size.set_alignment(0, 0.5, 0, 0)
+        self.picker_font_size.set_title(_("Default font size"))
+        self.vbox.add(self.picker_font_size)
+        self.picker_font_size.show()
+
+        # Load images check button
+        self.auto_load_images = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.auto_load_images.set_label(_('Auto load images'))
+        self.vbox.add(self.auto_load_images)
+        self.auto_load_images.show()
+
+        # Load settings
+        self.auto_load_images.set_active(settings.auto_load_images)
+        self.selector_orientation.set_active(0, settings.rotation_mode)
+        try:
+            font_size_index = constants.font_size_range.index(settings.default_font_size)
+        except ValueError:
+            # defaults to 16pt
+            font_size_index = constants.font_size_range.index(16)
+        self.selector_font_size.set_active(0, font_size_index)
+
 class NewFeedDialog(gtk.Dialog):
 
     def __init__(self, parent):
@@ -234,8 +274,8 @@ class FeedsWindow(hildon.StackableWindow):
         app_version = constants.RSS_VERSION
 
         # Auto rotate
-        rotation_object = FremantleRotation(app_name, self,
-                                                app_version, FremantleRotation.AUTOMATIC)
+        self.rotation_object = FremantleRotation(app_name, self, app_version,
+                                                 FremantleRotation.AUTOMATIC)
 
         # Edit toolbar
         self.edit_toolbar_button_handler = 0
@@ -259,6 +299,12 @@ class FeedsWindow(hildon.StackableWindow):
         self.connect('delete-event', self._on_exit)
         hildon.Program.get_instance().add_window(self)
 
+        # Apply settings
+        settings.load()
+        self.view.sort(settings.feeds_order)
+        if settings.feeds_order == constants.DESCENDING_ORDER:
+            self.descending_filter_button.set_active(True)
+
         # Load Feeds (could be none)
         try:
             manager.load(self._feed_data_loaded)
@@ -294,6 +340,7 @@ class FeedsWindow(hildon.StackableWindow):
     def _on_exit(self, window, event):
         # Save the feeds status
         manager.save(self._on_feed_data_saved)
+        settings.save()
 
     def _on_feed_data_saved(self, user_data):
         # Close application
@@ -302,6 +349,20 @@ class FeedsWindow(hildon.StackableWindow):
     def _create_menu(self):
         menu = hildon.AppMenu()
 
+        # Sorting filter
+        self.ascending_filter_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.ascending_filter_button.set_mode(False)
+        self.ascending_filter_button.set_label(_('A-Z'))
+        self.ascending_filter_button.connect('clicked', self._sort_ascending_cb)
+        menu.add_filter(self.ascending_filter_button)
+        self.descending_filter_button = \
+            hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                  group = self.ascending_filter_button)
+        self.descending_filter_button.set_mode(False)
+        self.descending_filter_button.set_label(_('Z-A'))
+        self.descending_filter_button.connect('clicked', self._sort_descending_cb)
+        menu.add_filter(self.descending_filter_button)
+
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('New Feed'))
         button.connect('clicked', self._new_feed_cb)
@@ -317,11 +378,10 @@ class FeedsWindow(hildon.StackableWindow):
         button.connect('clicked', self._update_all_cb)
         menu.append(button)
 
-        # Disabled for the moment as we don't handle settings yet
-        # button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        # button.set_label(_('Settings'))
-        # button.connect('clicked', self._preferences_cb)
-        # menu.append(button)
+        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        button.set_label(_('Settings'))
+        button.connect('clicked', self._preferences_cb)
+        menu.append(button)
 
         menu.show_all()
         self.set_app_menu(menu)
@@ -383,7 +443,7 @@ class FeedsWindow(hildon.StackableWindow):
         self.unfullscreen()
 
     def _preferences_cb(self, button):
-        dialog = PreferencesDialog(self)
+        dialog = SettingsDialog(self)
         dialog.connect('response', self._preferences_response_cb)
         dialog.show_all()
 
@@ -445,9 +505,14 @@ class FeedsWindow(hildon.StackableWindow):
     def _preferences_response_cb(self, dialog, response):
 
         if response == gtk.RESPONSE_ACCEPT:
-            print 'save preferences'
+            settings.auto_load_images = dialog.auto_load_images.get_active()
+            settings.rotation_mode = dialog.selector_orientation.get_active(0)
+            settings.default_font_size = int(dialog.selector_font_size.get_current_text())
             dialog.destroy()
 
+            # Update rotation
+            self.rotation_object.set_mode(settings.rotation_mode)
+
     def _feed_data_loaded(self, user_data):
         # TODO: migrate to "with" statement when available
         try:
@@ -475,6 +540,14 @@ class FeedsWindow(hildon.StackableWindow):
             dialog.connect('response', self._new_feed_response_cb)
             dialog.show_all()
 
+    def _sort_ascending_cb(self, button):
+        settings.feeds_order = constants.ASCENDING_ORDER
+        self.view.sort(constants.ASCENDING_ORDER)
+
+    def _sort_descending_cb(self, button):
+        settings.feeds_order = constants.DESCENDING_ORDER
+        self.view.sort(constants.DESCENDING_ORDER)
+
 class EntriesView(hildon.GtkTreeView):
 
     ENTRIES_ICON_COLUMN = 0
@@ -519,9 +592,9 @@ class EntriesView(hildon.GtkTreeView):
             title = entry.title
 
         if 'read' in entry and entry['read'] == True:
-            color = self.style.lookup_color(constants.SECONDARY_TEXT_COLOR).to_string()
+            color = secondary_color.to_string()
         else:
-            color = self.style.lookup_color(constants.ACTIVE_TEXT_COLOR).to_string()
+            color = active_color.to_string()
 
         renderer.set_property('markup',
                               '<span foreground="%s">%s</span>\n<span size="small">%s</span>'
@@ -650,23 +723,23 @@ class ItemWindowHeader(gtk.HBox):
         self.author = gtk.Label('')
         self.author.set_alignment(0,0.5)
         self.author.set_ellipsize(pango.ELLIPSIZE_END)
-        self.url = gtk.LinkButton('', _('View in Browser'))
-        self.url.set_alignment(0,0.5)
+        # self.url = gtk.LinkButton('', _('View in Browser'))
+        # self.url.set_alignment(0,0.5)
         header_vbox.pack_start(self.author, True, False)
-        header_vbox.pack_start(self.url, True, False)
+#        header_vbox.pack_start(self.url, True, False)
         self.author.show()
-        self.url.show()
+#        self.url.show()
         self.set_item(item)
         header_vbox.show()
 
         # Navigation buttons. Sizes are important. We set the size of the icon but
         # we let the button grow as needed
-        self.button_up = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        self.button_up = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button_up_img = gtk.Image()
         button_up_img.set_from_icon_name(constants.ICON_UP_NAME,
                                               gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.button_up.set_image(button_up_img)
-        self.button_down = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        self.button_down = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button_down_img = gtk.Image()
         button_down_img.set_from_icon_name(constants.ICON_DOWN_NAME,
                                                 gtk.HILDON_SIZE_FINGER_HEIGHT)
@@ -680,7 +753,7 @@ class ItemWindowHeader(gtk.HBox):
         self.pack_start(navigation_hbox, False, False)
 
         # Signals
-        self.url.connect('clicked', self._link_button_clicked)
+        #self.url.connect('clicked', self._link_button_clicked)
 
     def _link_button_clicked(self, link_button):
         print 'Open ' + link_button.get_uri()
@@ -693,7 +766,7 @@ class ItemWindowHeader(gtk.HBox):
         item['read'] = True
 
         self.author.set_text(get_author_from_item(item))
-        self.url.set_uri(item.link)
+        #self.url.set_uri(item.link)
 
 class ItemWindow(hildon.StackableWindow):
 
@@ -713,34 +786,36 @@ class ItemWindow(hildon.StackableWindow):
         self.item_window_header.button_down.connect("clicked", self._down_button_clicked)
 
         # HTML renderer
-        self.view = gtkhtml2.View()
-        self.document = gtkhtml2.Document()
-        self.view.set_document(self.document)
-        self.document.connect('request-url', self._on_request_url)
+        self.view = webkit.WebView()
+        wbk_settings = self.view.get_settings()
+        # TODO: read this from settings
+        wbk_settings.set_property('default_font_size', settings.default_font_size)
+        wbk_settings.set_property('auto-load-images', settings.auto_load_images)
+        wbk_settings.set_property('auto-shrink-images', True)
         self.set_item(item)
 
-        # Alignment for padding
-        align = gtk.Alignment(0, 0, 1, 1)
-        align.set_padding(4, 0 , 16, 16)
-
         # Pannable for showing content
         pannable = hildon.PannableArea()
         pannable.add(self.view)
         pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
+        pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
         self.view.show()
 
+        # Header Alignment
+        header_align = gtk.Alignment(0, 0, 1, 1)
+        header_align.set_padding(4, 0 , 16, 16)
+        header_align.add(self.item_window_header)
+        header_align.show()
+
         vbox = gtk.VBox(False, 8)
-        vbox.pack_start(self.item_window_header, False, False)
+        vbox.pack_start(header_align, False, False)
         vbox.pack_start(pannable, True, True)
         self.item_window_header.show()
         pannable.show()
 
-        align.add(vbox)
+        self.add(vbox)
         vbox.show()
 
-        self.add(align)
-        align.show()
-
         self._create_menu()
 
     def _on_request_url(self, object, url, stream):
@@ -788,14 +863,13 @@ class ItemWindow(hildon.StackableWindow):
         pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
 
         item = self.item_window_header.item
-        color = self.style.lookup_color(constants.SECONDARY_TEXT_COLOR).to_string()
 
         row = 0;
         table = gtk.Table()
         table.set_col_spacing(0, 12)
-        self._add_to_table (table, color, _('Title'), item.title, row)
+        self._add_to_table (table, secondary_color.to_string(), _('Title'), item.title, row)
         row += 1
-        self._add_to_table (table, color, _('Published'),
+        self._add_to_table (table, secondary_color.to_string(), _('Published'),
                             time.strftime('%x', item.updated_parsed), row)
 
         pannable.add_with_viewport(table)
@@ -855,15 +929,8 @@ class ItemWindow(hildon.StackableWindow):
             # Should never happen
             body = _('No text')
 
-        # unicode -> encoding -> latin-1. In case of characters not belonging to
-        # latin-1 do xmlcharrefreplace
-        body = body.decode(self.feed_data.encoding).encode('latin-1', 'xmlcharrefreplace')
-
         # Write HTML
-        self.document.clear()
-        self.document.open_stream(content_type)
-        self.document.write_stream("<html><body>%s</body></html>" % body)
-        self.document.close_stream()
+        self.view.load_string(body, content_type, self.feed_data.encoding, '')
 
         # Update button sensitiviness
         path = self.model.get_path(self.model_iter)
